# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" This module contains the base item class to represent an item of the API.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"


class LazyItemValue(object):
    """ Just to represent a value for an attribute of the BaseItem to fetch
    data only when is necessary.
    """


class BaseItem(object):
    """ This class is to represent an item of the API. The following attributes
    may be provided in the children classes:
     - extra_attributes: usually the api doesn't fetch the entire the data
                         related to an item, so we determine which are the
                         remaining attributes to be fetched lately once
                         the attribute is triggered.
     - translate: some attributes in the api are not consistent along different
                  service data, so use this dict to translate attributes coming
                  from api to consistent ones.
    """
    extra_attributes = []
    translate = {}
    aliases_fix = {}

    def __init__(self, resource, **kwargs):
        """ It requires the correspond resource instance and generic kwargs
        to set this item attributes. The "extra_attibutes" are being set as
        a LazyItemValue() instance firstly. Afterwards, the __getattribute__
        method may lately return the proper value coming from the API.
        The "alias" attribute are not always present, so is saves as a cache
        value _alias in case is present, otherwise keep it as None. The "alias"
        property in this class provides the value properly.
        """
        self.resource = resource
        self._extra_data = {}
        self.name = None
        self._alias = None
        for key in self.extra_attributes:
            setattr(self, key.lower(), LazyItemValue())
        for key, value in kwargs.items():
            attr = key.lower()
            if attr == 'alias':
                setattr(self, '_alias', value)
                continue
            if attr in self.translate:
                attr = self.translate[attr]
            setattr(self, attr, value)

    def __str__(self):
        """ Just the item name.
        """
        return str(self.name)

    def __repr__(self):
        """ The object representation.
        """
        return "<%s: %s>" % (self.__class__.__name__, str(self))

    def __getattribute__(self, item):
        """ It checks for LazyItemValue value to get the value properly because
        the information got from the API server was incomplete, (usually
        because list services only provides a few necessary data, more
        complete data must be taken from "get" services).
        """
        value = super(BaseItem, self).__getattribute__(item)
        if isinstance(value, LazyItemValue):
            d = super(BaseItem, self).__getattribute__('__dict__')
            resource = d['resource']
            data = resource.db.get_data(resource.get_path, id=d['id'])
            for key, value in data['Data'][resource.name].items():
                setattr(self, key.lower(), value)
        value = super(BaseItem, self).__getattribute__(item)
        if isinstance(value, LazyItemValue):
            setattr(self, item, None)
        return super(BaseItem, self).__getattribute__(item)

    @property
    def alias(self):
        """ If the _alias cache is None, just build the alias from the item
        name.
        """
        if self._alias is None:
            if self.name in self.aliases_fix:
                self._alias = self.aliases_fix[self.name]
            else:
                self._alias = self.name.lower()\
                                  .replace(' ', '-')\
                                  .replace('(', '')\
                                  .replace(')', '')
        return self._alias



# TheGamesDb API, Python Wrapper
# http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" This is the main module containing the main class TheGamesDb that provides
the resources and its methods to retrieve games related data through the
Games DB API (http://wiki.thegamesdb.net/index.php/Main_Page).
"""
__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"
__version__ = "0.1"
__maintainer__ = "Rogerio Hilbert Lima"
__email__ = "rogerhil@gmail.com"

import time
from urllib.parse import urlencode
from urllib.request import Request, urlopen
from urllib.error import HTTPError
from random import randint
from xmltodict import parse

from .resources import GameResource, PlatformResource


USER_AGENTS = [
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36"
    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:23.0) Gecko/20100101 Firefox/23.0',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36',
    'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; WOW64; Trident/6.0)',
    'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.146 Safari/537.36',
    'Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.146 Safari/537.36',
    'Mozilla/5.0 (X11; Linux x86_64; rv:24.0) Gecko/20140205 Firefox/24.0 Iceweasel/24.3.0',
    'Mozilla/5.0 (Windows NT 6.2; WOW64; rv:28.0) Gecko/20100101 Firefox/28.0',
    'Mozilla/5.0 (Windows NT 6.2; WOW64; rv:28.0) AppleWebKit/534.57.2 (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2',
]


class TheGamesDb(object):
    """ The main class for The Games Db API. It provides the following
    resources at the momment:

    >>> api = TheGamesDb()
    >>> api.game
    <GameResource get,list>
    >>> api.platform
    <PlatformResource get,list,games>
    """

    base_url = 'http://thegamesdb.net/api/'
    base_images_url = 'http://thegamesdb.net/banners/'

    def __init__(self, request_attempts=10, seconds_between_attempts=5):
        """ This constructor sets the resources instances and the user
        agents to avoid to being banned by the API server. Every last request
        made by the API saves the last response in the self.last_response.
        If the request fails because a 502 status, it attempts to do it again
        many times defined in the request_attempts argument. And the attempt is
        made after seconds defined in the seconds_between_attempts argument.
        """
        self.game = GameResource(self)
        self.platform = PlatformResource(self)
        self.user_agents = USER_AGENTS
        self.last_response = None
        self.request_attempts = abs(request_attempts)
        self.seconds_between_attempts = abs(seconds_between_attempts)

    def _get_response(self, url, **params):
        """ Giving a service path and optional specific arguments, returns
        the response string.
        """
        data = urlencode(params)
        url = "%s?%s" % (url, data)
        headers = {'User-Agent': self.get_random_agent()}
        request = Request(url, headers=headers, method='GET')

        def open_request(request, attempts, err=None):
            if attempts > self.request_attempts:
                raise err
            attempts += 1
            try:
                with urlopen(request) as response:
                    return response.read()
            except HTTPError as err:
                if err.getcode() != 502:
                    raise err
                print("HTTPError occurred while trying to request the url "
                      "%s. %s. Trying again in %s seconds..." % (url, err,
                                                self.seconds_between_attempts))
                time.sleep(self.seconds_between_attempts)
                return open_request(request, attempts, err)

        attempts = 0
        self.last_response = open_request(request, attempts)
        return self.last_response

    def get_response(self, path, **params):
        """ Giving a service path and optional specific arguments, returns
        the response string.
        """
        url = "%s%s" % (self.base_url, path)
        return self._get_response(url, **params)

    def get_image_data(self, path):
        url = "%s%s" % (self.base_images_url, path)
        return self._get_response(url)

    def get_data(self, path, **params):
        """ Giving a service path and optional specific arguments, returns
        the XML data from the API parsed as a dict structure.
        """
        xml = self.get_response(path, **params)
        try:
            return parse(xml)
        except Exception as err:
            print(path)
            print(params)
            print(err)
            raise err

    def get_random_agent(self):
        """ Randomly returns one of the items in the the user_agents list
        defined.
        """
        return self.user_agents[randint(0, len(self.user_agents) - 1)]



# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
""" This module contains the an implementation of the main class TheGamesDb
mocked, for testing purposes, to read XML files instead of requesting URLs.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"

import os
from urllib.parse import urlencode

from ..api import TheGamesDb

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

class MockException(Exception):
    pass


class TheGamesDbMock(TheGamesDb):
    """ It inherits from TheGamesDb to especially overrides the get_response
    method to read XML files saved locally instead of using the network.
    """

    def __init__(self, xml_path=None):
        """ Overrides just to include the options argument xml_path.
        """
        super(TheGamesDbMock, self).__init__()
        self.xml_path = XmlsDb.xml_path if xml_path is None else xml_path

    def get_response(self, path, **params):
        """ Each XML file save has the exact service path name with the
        parameters, so it's easy to get the proper by checking the file names.
        If the the service path + query string doesn't match any XML file, it
        raises the MockException.
        """
        service = XmlsDb.get_service_path_by_path_params(path, **params)
        response = None
        for filename in os.listdir(self.xml_path):
            if service == os.path.splitext(filename)[0]:
                with open(os.path.join(self.xml_path, filename)) as xml:
                    response = xml.read()
        if response is None:
            raise MockException('No mock xml file for "%s"' % service)
        return response


class XmlsDb(object):

    xml_path = os.path.join(BASE_DIR, 'test/xml')

    def __init__(self):
        self.api = TheGamesDb()

    @staticmethod
    def get_service_path_by_path_params(path, **params):
        params = dict([(k, v) for k, v in params.items() if v])
        qs = ("?%s" % urlencode(params) if params else "")
        service = "%s%s" % (path, qs)
        return service

    @classmethod
    def save_xml(cls, data, path, **params):
        service = cls.get_service_path_by_path_params(path, **params)
        with open(os.path.join(cls.xml_path, "%s.xml" % service), 'w') as xml:
            xml.write(data.decode('utf-8'))

    def update_platforms_xmls(self):
        platforms = self.api.platform.list()
        self.save_xml(self.api.last_response, self.api.platform.list_path)
        for platform in platforms:
            self.api.platform.get(platform.id)
            platform_name_plus = platform.name.lower()
            self.save_xml(self.api.last_response, self.api.platform.get_path,
                          id=platform.id)
            platform.games()
            self.save_xml(self.api.last_response, self.api.platform.games_path,
                          platform=platform_name_plus)

    def update_games_xmls(self):
        games = self.api.game.list(name='x-men')
        self.save_xml(self.api.last_response, self.api.game.list_path,
                      name='x-men')
        for game in games:
            self.api.game.get(game.id)
            self.save_xml(self.api.last_response, self.api.game.get_path,
                          id=game.id)
        games = self.api.game.list(name='Origins')
        self.save_xml(self.api.last_response, self.api.game.list_path,
                      name='Origins')
        for game in games:
            self.api.game.get(game.id)
            self.save_xml(self.api.last_response, self.api.game.get_path,
                          id=game.id)




# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" Define the classes to represent the items returned from this api.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"

from .item import BaseItem


class Game(BaseItem):
    """ The Game item must translate "gametitle" coming from the XML to "name"
    to keep consistency/
    """
    extra_attributes = [
        'releasedate', 'platform', 'overview', 'esrb', 'genres', 'players',
        'co-op', 'youtube', 'publisher', 'developer', 'rating', 'similar',
        'images'
    ]
    translate = {'gametitle': 'name', 'co-op': 'co_op', 'thumb': '_thumb'}

    base_img_url = "http://thegamesdb.net/banners"
    boxart_base = "%s/boxart/original" % base_img_url
    boxart_thumb_base = "%s/boxart/thumb/original" % base_img_url

    def __str__(self):
        """ Overrides to include the platform.
        """
        return "%s (%s)" % (str(self.name), self.platform)

    @property
    def thumb(self):
        if not getattr(self, '_thumb', None):
            if self.images is None:
                return
            boxart = self.images.get('boxart')
            if boxart is None:
                return
            if isinstance(boxart, list):
                boxart = boxart[0]
            self._thumb = boxart['@thumb']
        return self._thumb

    @property
    def boxart_front(self):
        if self.thumb is None:
            return
        return "%s/front/%s" % (self.boxart_base, self.thumb.split('/')[-1])

    @property
    def boxart_back(self):
        if self.thumb is None:
            return
        return "%s/back/%s" % (self.boxart_base, self.thumb.split('/')[-1])

    @property
    def boxart_front_thumb(self):
        if self.thumb is None:
            return
        return "%s/front/%s" % (self.boxart_thumb_base,
                                self.thumb.split('/')[-1])

    @property
    def boxart_back_thumb(self):
        if self.thumb is None:
            return
        return "%s/back/%s" % (self.boxart_thumb_base,
                               self.thumb.split('/')[-1])


class Platform(BaseItem):
    """ The Platform item must translate "platform" attribute coming from the
    XML to "name" to keep consistency.
    """
    extra_attributes = [
        'console', 'controller', 'overview', 'developer', 'manufacturer',
        'cpu', 'memory', 'graphics', 'sound', 'display', 'media',
        'maxcontrollers', 'rating', 'images'
    ]
    translate = {'platform': 'name'}
    aliases_fix = {
        'Nintendo Game Boy': 'nintendo-gameboy',
        'Nintendo Game Boy Advance': 'nintendo-gameboy-advance',
        'Nintendo Game Boy Color': 'nintendo-gameboy-color',
    }

    def games(self):
        """ This resource provides this extra method to retrieve all the games
        for this platform.
        """
        return self.resource.games(self.name)



# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" This module contains the resources classes used in the main TheGamesDb
class.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"

from .base import Resource
from .items import Game, Platform


class GameResource(Resource):
    """ Game resource.
    """
    name = 'Game'
    get_path = 'GetGame.php'
    list_path = 'GetGamesList.php'
    item_class = Game

    def list(self, name, platform='', genre=''):
        """ The name argument is required for this method as per the API
        server specification. This method also provides the platform and genre
        optional arguments as filters.
        """
        data_list = self.db.get_data(self.list_path, name=name,
                                     platform=platform, genre=genre)
        data_list = data_list.get('Data') or {}
        games = data_list.get('Game') or []
        return [self._build_item(**i) for i in games]


class PlatformResource(Resource):
    """ Platform resource.
    """
    name = 'Platform'
    get_path = 'GetPlatform.php'
    list_path = 'GetPlatformsList.php'
    games_path = 'PlatformGames.php'
    item_class = Platform

    def __repr__(self):
        """ This method is overrode to include the "games" method in the object
        representation.
        """
        return "<%s get,list,games>" % self.__class__.__name__

    def list(self):
        """ No argument is required for this method as per the API server
        specification.
        """
        data_list = self.db.get_data(self.list_path)
        data_list = data_list.get('Data') or {}
        platforms = (data_list.get('Platforms') or {}).get('Platform') or []
        return [self._build_item(**i) for i in platforms]

    def games(self, platform):
        """ It returns a list of games given the platform *alias* (usually is
        the game name separated by "-" instead of white spaces).
        """
        platform = platform.lower()
        data_list = self.db.get_data(self.games_path, platform=platform)
        data_list = data_list.get('Data') or {}
        return [Game(self.db.game, **i) for i in data_list.get('Game') or {}]



# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
""" Tests implementation.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"

from unittest import TestCase

from .mockdb import TheGamesDbMock
from ..items import Game, Platform


class BaseTest(TestCase):

    def setUp(self):
        self.api = TheGamesDbMock()


class TestGame(BaseTest):

    def test_list(self):
        games = self.api.game.list("Origins")
        for game in games:
            self._test_individual_game(game)

    def test_get(self):
        game = self.api.game.get(49)
        self._test_individual_game(game)
        game = self.api.game.get(4209)
        self._test_individual_game(game)

    def _test_individual_game(self, game):
        self.assertIsInstance(game, Game)
        self.assertIsNotNone(getattr(game, 'id', None))
        self.assertIsNotNone(getattr(game, 'name', None))
        if getattr(game, 'releasedate') is None:
            print("Game (%s): the attribute releasedate is None" % game)
        #self.assertIsNotNone(getattr(game, 'releasedate', None),
        #                 "%s - %s" % (game, 'releasedate'))
        self.assertFalse(getattr(game, 'platform', None) is None)
        for attr in Game.extra_attributes:
            self.assertTrue(hasattr(game, attr),
                            "%s - %s" % (game, attr))
            if getattr(game, attr) is None:
                print("Game (%s): the attribute %s is None" % (game, attr))
            #self.assertFalse(getattr(game, attr) is None,
            #                 "%s - %s" % (game, attr))


class TestPlatform(BaseTest):

    def test_list(self):
        platforms = self.api.platform.list()
        for platform in platforms:
            self._test_individual_platform(platform)

    def test_get(self):
        platform = self.api.platform.get(15)
        self._test_individual_platform(platform)

    def _test_individual_platform(self, platform):
        self.assertIsInstance(platform, Platform)
        self.assertIsNotNone(getattr(platform, 'id', None))
        self.assertIsNotNone(getattr(platform, 'name', None))
        self.assertIsNotNone(getattr(platform, 'alias', None))
        for attr in Platform.extra_attributes:
            self.assertTrue(hasattr(platform, attr),
                            "%s - %s" % (platform, attr))
            if getattr(platform, attr) is None:
                print("Platform (%s): the attribute %s is None"
                      % (platform, attr))
            #self.assertIsNotNone(getattr(platform, attr),
            #                     "%s - %s" % (platform, attr))

    def test_aliases(self):
        platforms = self.api.platform.list()
        for platform in platforms:
            platform_from_get = self.api.platform.get(platform.id)
            self.assertEquals(platform.alias, platform_from_get.alias,
                              "The platform id is: %s" % platform.id)



import os
from setuptools import setup, find_packages

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

local_file = lambda f: open(os.path.join(os.path.dirname(__file__), f)).read()

if __name__ == '__main__':
    setup(
        name='thegamesdb',
        version='0.5',
        description='The Games DB API wrapper for Python',
        long_description=long_description,
        author='Rogerio Hilbert Lima',
        author_email='rogerhil@gmail.com',
        url='https://github.com/rogerhil/thegamesdb',
        download_url='https://github.com/rogerhil/thegamesdb/tarball/0.5',
        packages=find_packages()
    )



# TheGamesDb API, Python Wrapper - http://wiki.thegamesdb.net/index.php/Main_Page
# Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

""" This is the base module that contains the base abstract class for a generic
game resource.
"""

__author__ = "Rogerio Hilbert Lima"
__copyright__ = "Copyright (C) 2015 Rogerio Hilbert Lima <rogerhil@gmail.com>"
__license__ = "GPL"


class Resource(object):
    """ Abstract class for a generic game resource. Child classes must provide
    the following attributes:
     - name: the resource name based on the XML tag,
             e.g.: <Platform></Platform> => Platform.
     - get_path: the path to get an item,, e.g.: GetGame.php
     - list_path: the path to list items,, e.g.: GetGamesList.php
     - item_class: a class that represents and item for this resource.
    """
    name = None
    get_path = None
    list_path = None
    item_class = None

    def __init__(self, db):
        """ It requires the main TheGamesDb object.
        """
        self.db = db

    def __str__(self):
        """ Just this resource name.
        """
        return str(self.name)

    def __repr__(self):
        """ The object string representation includes the important methods of
        this resource.
        """
        return "<%s get,list>" % self.__class__.__name__

    def _build_item(self, **data):
        """ Instantiate an item using the item_class defined in the class
        passing this resource as the first argument and the correspond data.
        """
        return self.item_class(self, **data)

    def get(self, id):
        """ Gets the dict data and builds the item object.
        """
        data = self.db.get_data(self.get_path, id=id)
        return self._build_item(**data['Data'][self.name])

    def list(self, **kwargs):
        """ This method is abstract and needs to be implemented because the
        kwargs might be different per resource.
        """
        raise NotImplementedError
